// $Id: mailbox.cc,v 1.7 2000/11/14 23:38:55 classic Exp $
//
// Implementation of the Mailbox class
// (c) by Torben Nehmer 2000
// torben@nehmer.net

#include"mailbox.h"

Mailbox::Mailbox (string strServer, int iPort) {
  if (!connect (strServer, iPort))
    exit(1);
}

bool Mailbox::connect (string strServer, int iPort) {
  string data;
  int x,y;
  m_sockIMAP.connect (strServer, iPort);

  data = m_sockIMAP.readline();

  if (data.find("* OK") == -1) {
    cout << "There was no OK response from the server. Aborted\n";
    return false;
  }

  // Requesting capabilities
  m_sockIMAP.write("0 CAPABILITY\n");

  data = m_sockIMAP.readline();

#ifdef DEBUG
  cout << data;
#endif

  string::iterator iter1 = data.begin();
  string::iterator iter2;
  // Ignore "* CAPABILITY "
  advance (iter1, 13);
  iter2 = iter1;

  while (iter2 != data.end()) {
    while ( (*iter2) != ' ' && iter2 != data.end() ) 
      iter2++;

    if (iter2 == data.end()) {
      m_vCapabilities.push_back (string (iter1, iter2 -1));
      break;
    } else {
      m_vCapabilities.push_back (string (iter1, iter2));
      iter2++;
      iter1 = iter2;
    }
  }

#ifdef DEBUG
  cout << m_vCapabilities.size() << " capabilities found: \n";
  copy(m_vCapabilities.begin(),
       m_vCapabilities.end(),
       ostream_iterator<string>(cout, "\n"));
#endif

  data = m_sockIMAP.readline();

#ifdef DEBUG
  cout << data;
#endif
  if (data.find("0 OK CAPABILITY") < 0) {
    cout << "Fatal error: Didn't got OK response. Aborted\n";
    return false;
  }

  m_iState = 0;
  m_strMailbox = "";

  return true;
}


bool Mailbox::login (string strUser, string strPassword) {
  if (m_iState == 0) {
    string data;
#ifdef DEBUG
    cout << "Sending: " << ("1 LOGIN " + strUser + " \"" + strPassword + "\"\n");
#endif
    m_sockIMAP.write("1 LOGIN " + strUser + " \"" + strPassword + "\"\n");

    data = m_sockIMAP.readline();
#ifdef DEBUG
    cout << data;
#endif

    if (data.find("1 OK LOGIN") != -1) {
      m_iState = 1;
      return true;
    } else {
      m_iState = 0;
      return false;
    }
  } else {
#ifdef DEBUG
    cout << "Already logged in. Aborting login\n";
#endif
  }
}

Mailbox::Mailbox (string strServer,
		  string strUser,
		  string strPassword,
		  int iPort) {
  if (!connect (strServer, iPort))
    exit (1);
  login (strUser, strPassword);
}

Mailbox::mbx_status * Mailbox::examine (string strMailbox) {
  mbx_status * pResult;
#ifdef DEBUG
  cout << "Sending: " << ("2 EXAMINE " + strMailbox + "\n");
#endif
  m_sockIMAP.write ("2 EXAMINE " + strMailbox + "\n");

  pResult = parse_select(false);
  
  if (pResult) {
    m_iState = 2;
    m_strMailbox = strMailbox;
    m_bMailboxReadWrite = false;
  }

  return pResult;
}

Mailbox::mbx_status * Mailbox::parse_select(bool bSelect) {
  string command = (bSelect ? "SELECT" : "EXAMINE");
  string data;
  string::iterator iter1, iter2;
  mbx_status * pResult;
  

  data = m_sockIMAP.readline();

#ifdef DEBUG
  cout << "got command " << command << endl;
  cout << data;
#endif
  
  if (data.find(" NO ") != -1) {
    cout << "Mailbox does not exist!\n";
    return NULL;
  }

  pResult = new mbx_status;
#ifdef DEBUG
  pResult->iExists = -1;
  pResult->iRecent = -1;
#endif;
  pResult->iUnseen = -1;
  pResult->iUIDValidity = -1;
  pResult->iUIDNext = -1;
  
  while (data.find(" OK ") == -1 || data.find(command) == -1) {
    if (data.find ("EXISTS") != -1) {
      // EXISTS response: * 107 EXISTS
#ifdef DEBUG
      cout << "Got Exists\n";
#endif
      iter1 = data.begin();
      advance(iter1, 2);
      iter2 = iter1;
      while (*iter2 != ' ') iter2++;
      pResult->iExists = atol(string(iter1, iter2).c_str());
    } else if (data.find ("RECENT") != -1) {
      // RECENT response: * 1 RECENT
#ifdef DEBUG
      cout << "Got Recent\n";
#endif
      iter1 = data.begin();
      advance(iter1, 2);
      iter2 = iter1;
      while (*iter2 != ' ') iter2++ ;
      pResult->iRecent = atol(string(iter1, iter2).c_str());
    } else if (data.find ("UIDVALIDITY") != -1) {
#ifdef DEBUG
      cout << "Got UID validity value\n";
#endif
      iter1 = data.begin();
      advance (iter1, 18);
      iter2 = iter1;
      while (*iter2 != ']') iter2++ ;
      pResult->iUIDValidity = atol(string(iter1, iter2).c_str());
    } else if (data.find("UIDNEXT") != -1) {
#ifdef DEBUG
      cout << "Got next UID\n";
#endif
      iter1 = data.begin();
      advance (iter1, 14);
      iter2 = iter1;
      while (*iter2 != ']') iter2++;
      pResult->iUIDNext = atol(string(iter1, iter2).c_str());
    } else if (data.find(" FLAGS") != -1) {
#ifdef DEBUG
      cout << "Got Flags\n";
#endif
      iter1 = data.begin();
      while (*iter1 != ')') {
	while (*iter1 != '\\' && *iter1 != ')') iter1++;
	iter2 = iter1;
	while (*iter2 != ' ' && *iter2 != ')') iter2++;
	if (iter1 != iter2)
	  pResult->vFlags.push_back(string(iter1 + 1, iter2));
	iter1 = iter2;
      }
    } else if (data.find("PERMANENTFLAGS") != -1) {
#ifdef DEBUG
      cout << "Got Permanent Flags\n";
#endif
      iter1 = data.begin();
      while (*iter1 != ')') {
	while (*iter1 != '\\' && *iter1 != ')') iter1++;
	iter2 = iter1;
	while (*iter2 != ' ' && *iter2 != ')') iter2++;
	if (iter1 != iter2)
	  pResult->vPermflags.push_back(string(iter1 + 1, iter2));
	iter1 = iter2;
      }
    } else if (data.find("UNSEEN") != -1) {
#ifdef DEBUG
      cout << "Got Unseen\n";
#endif
      iter1 = data.begin();
      advance(iter1, 13);
      iter2 = iter1;
      while (*iter2 != ']') iter2++;
      pResult->iUnseen = atol(string(iter1, iter2).c_str());
    }
      



    data = m_sockIMAP.readline();
#ifdef DEBUG
    cout << data;
#endif
  }
   
  return pResult;
    
}

Mailbox::mbx_status * Mailbox::select (string strMailbox) {
  mbx_status * pResult;
  

#ifdef DEBUG
  cout << "Sending: " << ("3 SELECT " + strMailbox + "\n");
#endif
  m_sockIMAP.write ("3 SELECT " + strMailbox + "\n");

  pResult = parse_select();
  if (pResult) {
    m_iState = 2;
    m_strMailbox = strMailbox;
    m_bMailboxReadWrite = true;
  }

  return pResult;
}

vector<long> Mailbox::search (string strQuery) {
  vector<long> vResult;
  string data;
  string::iterator iter1, iter2;

  if (m_iState != 2) {
#ifdef DEBUG
    cout << "No Mailbox selected. Aborting...\n";
#endif
    return vResult;
  }

#ifdef DEBUG
  cout << "Sending: " << ("4 SEARCH " + strQuery + "\n");
#endif
  m_sockIMAP.write ("4 SEARCH " + strQuery + "\n");

  data = m_sockIMAP.readline();
#ifdef DEBUG
  cout << data;
#endif

  if (data.find(" NO ") != -1) {
#ifdef DEBUG
    cout << "Search Query not successful, aborting...\n";
#endif
    return vResult;
  }
  iter1 = data.begin();
  advance(iter1, 9);
  while (*iter1 != '\n') {
    while (!isdigit(*iter1) && *iter1 != '\n') iter1++;
    iter2 = iter1;
    while (*iter2 != ' ' && *iter2 != '\n') iter2++;
    if (iter1 != iter2)
      vResult.push_back(atol(string(iter1, iter2).c_str()));
    iter1 = iter2;
  }

  data = m_sockIMAP.readline();
#ifdef DEBUG
  cout << data;
#endif
  if (data.find("4 OK SEARCH") == -1) {
#ifdef DEBUG
    cout << "Some obscure error occured...\n";
#endif
  }
  
  return vResult;
  
}

vector<long> Mailbox::search_recent() {
  return search("RECENT");
}

vector<long> Mailbox::search_unseen() {
  return search("UNSEEN");
}

Mailbox::msg_header * Mailbox::getheader (long UID) {
  string data = "";
  string::iterator iter1, iter2;
  char UIDbuffer[50];
  string strUID;
  msg_header * pResult = new msg_header;


  sprintf(UIDbuffer, "%i", UID);

#ifdef DEBUG
  cout << "Sending: "
       << ("5 FETCH " + string(UIDbuffer) +
	   " (BODY[HEADER.FIELDS (FROM TO SUBJECT DATE)])\n");
#endif
  m_sockIMAP.write ("5 FETCH " + string(UIDbuffer) +
		    " (BODY[HEADER.FIELDS (FROM TO SUBJECT DATE)])\n");


  do {
    data = m_sockIMAP.readline();
#ifdef DEBUG
    cout << data;
#endif
    if (data.find("5 BAD ") != -1 || data.find("5 NO ") != -1) {
      // Error in Request -> abort
      delete pResult;
      return NULL;
    } else if (data.find("To:") == 0) {
#ifdef DEBUG
      cout << "Got To:\n";
#endif
      iter1 = data.begin();
      advance(iter1, 4);
      iter2 = data.end();
      iter2--;
      pResult->strTo = string (iter1, iter2);
    } else if (data.find("Subject:") == 0) {
#ifdef DEBUG
      cout << "Got Subject:\n";
#endif
      iter1 = data.begin();
      advance(iter1, 9);
      iter2 = data.end();
      iter2--;
      pResult->strSubject = string (iter1, iter2);
    } else if (data.find("Date:") == 0) {
#ifdef DEBUG
      cout << "Got Date:\n";
#endif
      iter1 = data.begin();
      advance(iter1, 6);
      iter2 = data.end();
      iter2--;
      pResult->strDate = string (iter1, iter2);
    } else if (data.find("From:") == 0) {
#ifdef DEBUG
      cout << "Got From:\n";
#endif
      iter1 = data.begin();
      advance(iter1, 6);
      iter2 = data.end();
      iter2--;
      pResult->strFrom = string (iter1, iter2);
    } 
  } while (data.find("5 OK FETCH") == -1);

  return pResult;
 
}

void Mailbox::close() {
  string data;

#ifdef DEBUG
  cout << "Sending : 6 CLOSE\n";
#endif
  m_sockIMAP.write("6 CLOSE\n");

  data = m_sockIMAP.readline();
#ifdef DEBUG
  cout << data;
#endif
  // Ignore response. In any way, state will be "authenticated" after
  // a close.
  m_iState = 1;
  m_strMailbox = "";
}

void Mailbox::logout () {
#ifdef DEBUG
  cout << "Sending: 7 LOGOUT\n";
#endif
  m_sockIMAP.write ("7 LOGOUT\n");

#ifdef DEBUG
  string data;
  data = m_sockIMAP.readline();
  cout << data;
#endif;
  
  m_iState = 3;
}

Mailbox::~Mailbox () {
  logout();
}

int Mailbox::get_iState() {
  return m_iState;
}
