// $Id: socket.cc,v 1.4 2000/11/12 22:45:36 classic Exp $
//
// Implementation for socket communication class
// (c) by Torben Nehmer 2000
// torben@nehmer.net

// Include class header file
#include "socket.h"

// other includes
#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <algorithm>


const long Socket::MAX_READ_SIZE = 65536;

Socket::Socket () {
  m_sdSocketDescriptor = socket(AF_INET,SOCK_STREAM,0);

  // 2DO Exception
  if (m_sdSocketDescriptor < 0) {
    cout << "Failed creating socket. Aborting\n";
    exit (1);
  }
}

Socket::~Socket () {
  close (m_sdSocketDescriptor);
}

int Socket::listen (int iPort, int iBacklog) {
  m_addrLocal.sin_family = AF_INET;
  m_addrLocal.sin_port = htons(iPort);
  m_addrLocal.sin_addr.s_addr = INADDR_ANY;

  if (! ::bind(m_sdSocketDescriptor,
	     (const struct sockaddr *) &m_addrLocal,
	     sizeof(m_addrLocal)))
    return 1;

  ::listen (m_sdSocketDescriptor, iBacklog);
  return 0;
}

int Socket::connect (string strRemoteAddress, int iPort) {
  hostent * DNSresult;

  DNSresult = gethostbyname(strRemoteAddress.c_str());
  if (!DNSresult) 
    return -1;

  m_addrRemote.sin_family = AF_INET;
  m_addrRemote.sin_port = htons(iPort);
  // 2DO can this be done any better??? It looks rather error prone
  memcpy (&m_addrRemote.sin_addr.s_addr, 
	  DNSresult->h_addr, 
	  DNSresult->h_length);

  //delete DNSresult;

  // 2DO check 2nd argument type. 
  return ::connect(m_sdSocketDescriptor, 
		   (const struct sockaddr *) &m_addrRemote,
		   sizeof(m_addrRemote));
}

long Socket::write (string strData) {
  return ::write(m_sdSocketDescriptor,
	       strData.c_str(),
	       strData.size());
}

string Socket::read (long iCount) {
  char * buffer;
  int buffersize, bytesread;
  
  if (iCount > 0) {
    buffer = new char[iCount];
    buffersize = iCount;
  } else {
    buffer = new char[MAX_READ_SIZE];
    buffersize = MAX_READ_SIZE;
  }

  bytesread = ::read(m_sdSocketDescriptor,
		buffer,
		buffersize);

  char * start = buffer;
  char * end = buffer + bytesread;

  string data(start, end);
  
  delete[] buffer;

  return data;
}

int Socket::pollin (int iTimeout) {
  pollfd pfd;

  pfd.fd = m_sdSocketDescriptor;
  pfd.events = POLLIN;

  return ::poll(&pfd, 1, iTimeout);
}

int Socket::pollhup (int iTimeout) {
  pollfd pfd;

  pfd.fd = m_sdSocketDescriptor;
  pfd.events = POLLHUP;

  return ::poll(&pfd, 1, iTimeout);
}

string Socket::readline () {
  string recv = "";
  string data = "";

  while (recv != string("\n") && !pollhup(0) && pollin(1000)) {
    recv = read(1);
    data += recv;
  }

  return data;
}
